import 'dart:async';

import 'package:shared_preferences/shared_preferences.dart';

abstract class Setup {
  factory Setup.shared() {
    return _SharedPrefSetup.shared();
  }

  Future<bool> getBool(String key);

  Future<int> getInt(String key);

  Future<double> getDouble(String key);

  Future<String> getString(String key);

  Future<List<String>> getStringList(String key);

  Future<bool> putBool(String key, bool value);

  Future<bool> putInt(String key, int value);

  Future<bool> putDouble(String key, double value);

  Future<bool> putString(String key, String value);

  Future<bool> putStringList(String key, List<String> value);

  Future<bool> remove(String key);

  Future<bool> clear();
}

class _SharedPrefSetup implements Setup {
  const _SharedPrefSetup._();

  factory _SharedPrefSetup.shared() {
    return _instance;
  }

  static const _SharedPrefSetup _instance = _SharedPrefSetup._();

  @override
  Future<bool> getBool(String key) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.getBool(key);
  }

  @override
  Future<int> getInt(String key) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.getInt(key);
  }

  @override
  Future<double> getDouble(String key) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.getDouble(key);
  }

  @override
  Future<String> getString(String key) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.getString(key);
  }

  @override
  Future<List<String>> getStringList(String key) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.getStringList(key);
  }

  @override
  Future<bool> putBool(String key, bool value) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.setBool(key, value);
  }

  @override
  Future<bool> putInt(String key, int value) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.setInt(key, value);
  }

  @override
  Future<bool> putDouble(String key, double value) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.setDouble(key, value);
  }

  @override
  Future<bool> putString(String key, String value) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.setString(key, value);
  }

  @override
  Future<bool> putStringList(String key, List<String> value) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.setStringList(key, value);
  }

  @override
  Future<bool> remove(String key) async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.remove(key);
  }

  @override
  Future<bool> clear() async {
    SharedPreferences sharedPref = await SharedPreferences.getInstance();
    return sharedPref.clear();
  }
}
